
/*
   SimpleTex is a demo demonstrating texture mapping with
	 glide, it loads a texture from disk and projects
	 it on a triangle, can't be more simple!
*/

#include <glide.h>

void main(void)
{
  // hardware configuration structure
	GrHwConfiguration HwConfig;

  // Info about the 3df texture file
	Gu3dfInfo TexInfo;

  // Information about the texture, needed to be able
	// to download texture and configure the texture unit
	// for the texture
	GrTexInfo MipMap1;

	// texture memory startaddress on the TexelFx chip
 	FxU32 TexStartAddress;

	// the depth value;
  float w;

	// A triangle consists of 3 vertices
  GrVertex Triangle[3];

	// This is indices into the triangles vertices
  int vlist[3];

  // Initialise Glide - this function MUST be called
  // before anything else is done with Glide:
  grGlideInit();

  // Check for precence of 3dfx hardware:
  if ((grSstQueryHardware(&HwConfig)) == FXFALSE)
  {
        printf("Error! No 3dfx hardware!");
		exit(1);
  }
  // found 3dfx hardware!
  // select first voodoo board in system - this means you
  // can use different boards in your computer connected to
  // different monitors from the same program! Glide
  // only works with the current board selected from
  // the grSstSelect function.
  grSstSelect( 0 );


  // sets 3dfx videomode with some parameters
  if (grSstWinOpen( 0,
               GR_RESOLUTION_640x480,
               GR_REFRESH_60Hz,
               GR_COLORFORMAT_RGBA,
               GR_ORIGIN_UPPER_LEFT,
               2, 1) == FXFALSE)
  {
    printf("Error! Couldn't set 3dfx videomode, you need a voodoo board to run this demo!");
    exit(1);
  }


  /*
   Load mipmap file tex.3df from disk:
   Retrieve texture info, this is required before the
   texture is loaded because we must know how big the
   texture is so we can allocate the required memory for
   it.
  */

  // get the texture info:
  if (gu3dfGetInfo("tex.3df",&TexInfo) == FXTRUE)
  {
  	// allocate memory for the mipmap:
  	TexInfo.data = (void*)malloc(TexInfo.mem_required);
  	// Load the mipmap from disk:
  	gu3dfLoad("tex.3df",&TexInfo);
  }
  else
  {
		// Couldn't retrieve texture info, it's propably not
		// on disk!
  	grGlideShutdown();
		printf("Couldn't load tex.3df file!");
		exit(1);
  }

  /*
    Once texture is loaded into memory we have to download
    the textures to texture memory, usually we should perform
    some memory management but since this only is a
    simple demo program we just downloads the mipmap levels on
    the texture unit.
  */

  /*
    Take the information from the loaded mipmap and use it
    to set up a GrTexInfo structure which is used when we
    shall download the mipmap to texture memory on the
    texture units.
  */

	// We must setup the texture structure with these
	// values which was retrieved with Gu3dfGetInfo()
  MipMap1.smallLod = TexInfo.header.small_lod;
  MipMap1.largeLod = TexInfo.header.large_lod;
  MipMap1.aspectRatio = TexInfo.header.aspect_ratio;
  MipMap1.data = TexInfo.data;
  MipMap1.format = TexInfo.header.format;

  // download the mipmap levels to texture unit 1

  // we must find out the startaddresses of the texture memory on the
  // two texture units to be able to download the mipmaplevels on the
  // texture memory
  TexStartAddress = grTexMinAddress(GR_TMU0);

  // the actual downloading goes here
  grTexDownloadMipMap(GR_TMU0,TexStartAddress,
  	GR_MIPMAPLEVELMASK_BOTH,&MipMap1);


  // After the mipmap have been downloaded to texture memory
  // we must tell the texture units where to look for the
  // mipmap and how it is composed, we do this here:

  // tell texture unit 1 we have put the mipmaps at startaddress.
  grTexSource(GR_TMU0,TexStartAddress,GR_MIPMAPLEVELMASK_BOTH,
  	&MipMap1);


  // Now we must configure the TexelFx chip to output the texture
	// rgb colors to the color combine unit.
  grTexCombine(GR_TMU0,GR_COMBINE_FUNCTION_LOCAL,GR_COMBINE_FACTOR_NONE,
  	GR_COMBINE_FUNCTION_LOCAL,GR_COMBINE_FACTOR_NONE,
	FXFALSE,FXFALSE);

  // we must also configure the color combine unit to use the texture
	// rgb values as input to the color combine unit
  grColorCombine(GR_COMBINE_FUNCTION_SCALE_OTHER,
  	GR_COMBINE_FACTOR_ONE,GR_COMBINE_LOCAL_NONE,
	GR_COMBINE_OTHER_TEXTURE,FXFALSE);


	// Now we are ready to render the textured triangle:

  // set w to a valid value ( same as z )

	w = 100.0;

	// Setup the vertices for the textured triangle:
	// first the screen coordinates (x,y)
	Triangle[0].x = 100.0;
 	Triangle[0].y = 100.0;

	// Important!
	// The depth value is required for texture coordinates since
	// 3dfx hardware doesn't use normal texture coordinates (s,t) but
  // (sow,tow) which means that before s and t is written
  // to the GrVertex structure we needs to divide s by w
  // and t by w. This is required since 3dfx hardware is
	// using perspective correct texture mapping which means
	// that the hardware corrects for changes in z (w) and
	// to make perspective correct texture mapping you need
	// the z value (or oow here).

	Triangle[0].oow = 1/w;

	// then the texture coordinates - the texture coordinates spans
	// from 0.0/w to 256.0/w. If you specifies bigger values than
	// 256.0 then the texture starts to wrap around in the polygon.
	// for example: Writing 512.0 would display two textures of the
	// same kind in the polygon.

	Triangle[0].tmuvtx[0].sow = 0.0/w;
  Triangle[0].tmuvtx[1].sow = 0.0/w;
  Triangle[0].tmuvtx[0].tow = 0.0/w;
  Triangle[0].tmuvtx[1].tow = 0.0/w;
 	Triangle[0].tmuvtx[0].oow = 1/w;   // I think we need oow here to, not sure.
  Triangle[0].tmuvtx[1].oow = 1/w;


  Triangle[1].x = 500.0;
  Triangle[1].y = 100.0;
  Triangle[1].oow = 1/w;
  Triangle[1].tmuvtx[0].sow = 256.0/w;
  Triangle[1].tmuvtx[1].sow = 256.0/w;
  Triangle[1].tmuvtx[0].tow = 0.0/w;
  Triangle[1].tmuvtx[1].tow = 0.0/w;
  Triangle[1].tmuvtx[0].oow = 1/w;
  Triangle[1].tmuvtx[1].oow = 1/w;

  Triangle[2].x = 500.0;
  Triangle[2].y = 400.0;
  Triangle[2].oow = 1/w;
  Triangle[2].tmuvtx[0].sow = 256.0/w;
  Triangle[2].tmuvtx[1].sow = 256.0/w;
  Triangle[2].tmuvtx[0].tow = 128.0/w;
  Triangle[2].tmuvtx[1].tow = 128.0/w;
  Triangle[2].tmuvtx[0].oow = 1/w;
  Triangle[2].tmuvtx[1].oow = 1/w;

	// There values are indices into the triangle vertex
	// list, we need vlist for the grDrawPolygon routine
 	vlist[0] = 0;
	vlist[1] = 1;
	vlist[2] = 2;

  // clears the backbuffer RED
  grBufferClear( 0xFF000000, 0, GR_WDEPTHVALUE_FARTHEST );

	// FINALLY draw the textured triangle:
  grDrawPolygon(3,vlist,Triangle);

  // Make the triangle visible:
	grBufferSwap(1);

  // wait for a keypress:
	getch();

	// Shut down glide:
	grGlideShutdown();
}

